@@ -32,7 +32,6 @@
 
 #include <dbus/dbus-glib.h>
 
-#include <glade/glade-xml.h>
 #include <gconf/gconf-client.h>
 
 #include "gsd-a11y-preferences-dialog.h"
@@ -44,7 +43,7 @@
 
 #define GSD_A11Y_PREFERENCES_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_A11Y_PREFERENCES_DIALOG, GsdA11yPreferencesDialogPrivate))
 
-#define GLADE_XML_FILE "gsd-a11y-preferences-dialog.glade"
+#define GTKBUILDER_UI_FILE "gsd-a11y-preferences-dialog.ui"
 
 #define KEY_A11Y_DIR              "/desktop/gnome/accessibility"
 #define KEY_STICKY_KEYS_ENABLED   KEY_A11Y_DIR "/keyboard/stickykeys_enable"
@@ -72,10 +71,10 @@
 #define DPI_LOW_REASONABLE_VALUE 50
 #define DPI_HIGH_REASONABLE_VALUE 500
 
-#define DPI_LARGE_FONT   120
-#define DPI_LARGER_FONT  144
-#define DPI_LARGEST_FONT 192
-#define DPI_DEFAULT      96
+#define DPI_FACTOR_LARGE   1.25
+#define DPI_FACTOR_LARGER  1.5
+#define DPI_FACTOR_LARGEST 2.0
+#define DPI_DEFAULT        96
 
 #define KEY_GTK_THEME          "/desktop/gnome/interface/gtk_theme"
 #define KEY_COLOR_SCHEME       "/desktop/gnome/interface/gtk_color_scheme"
@@ -225,6 +224,21 @@ config_get_bool (const char *key,
         return enabled;
 }
 
+static double
+dpi_from_pixels_and_mm (int pixels,
+                        int mm)
+{
+        double dpi;
+
+        if (mm >= 1) {
+                dpi = pixels / (mm / 25.4);
+        } else {
+                dpi = 0;
+        }
+
+        return dpi;
+}
+
 static double
 get_dpi_from_x_server (void)
 {
@@ -233,10 +247,20 @@ get_dpi_from_x_server (void)
 
         screen = gdk_screen_get_default ();
         if (screen != NULL) {
-                dpi = gdk_screen_get_resolution (screen);
-                if (dpi < DPI_LOW_REASONABLE_VALUE || dpi > DPI_HIGH_REASONABLE_VALUE ||
-                    dpi < DPI_LOW_REASONABLE_VALUE || dpi > DPI_HIGH_REASONABLE_VALUE) {
+                double width_dpi;
+                double height_dpi;
+
+                width_dpi = dpi_from_pixels_and_mm (gdk_screen_get_width (screen),
+                                                    gdk_screen_get_width_mm (screen));
+                height_dpi = dpi_from_pixels_and_mm (gdk_screen_get_height (screen),
+                                                     gdk_screen_get_height_mm (screen));
+                if (width_dpi < DPI_LOW_REASONABLE_VALUE
+                    || width_dpi > DPI_HIGH_REASONABLE_VALUE
+                    || height_dpi < DPI_LOW_REASONABLE_VALUE
+                    || height_dpi > DPI_HIGH_REASONABLE_VALUE) {
                         dpi = DPI_DEFAULT;
+                } else {
+                        dpi = (width_dpi + height_dpi) / 2.0;
                 }
         } else {
                 /* Huh!?  No screen? */
@@ -252,25 +276,26 @@ config_get_large_print (gboolean *is_writable)
         gboolean     ret;
         GConfClient *client;
         GConfValue  *value;
-        gdouble      dpi;
+        gdouble      x_dpi;
+        gdouble      u_dpi;
 
         client = gconf_client_get_default ();
         value = gconf_client_get_without_default (client, KEY_FONT_DPI, NULL);
 
         if (value != NULL) {
-                dpi = gconf_value_get_float (value);
+                u_dpi = gconf_value_get_float (value);
                 gconf_value_free (value);
         } else {
-                dpi = get_dpi_from_x_server ();
+                u_dpi = DPI_DEFAULT;
         }
 
-        if (dpi < DPI_LOW_REASONABLE_VALUE) {
-                dpi = DPI_LOW_REASONABLE_VALUE;
-        }
+        x_dpi = get_dpi_from_x_server ();
 
         g_object_unref (client);
 
-        ret = (dpi > DPI_DEFAULT);
+        g_debug ("GsdA11yPreferences: got x-dpi=%f user-dpi=%f", x_dpi, u_dpi);
+
+        ret = (((double)DPI_FACTOR_LARGE * x_dpi) < u_dpi);
 
         return ret;
 }
@@ -283,7 +308,15 @@ config_set_large_print (gboolean enabled)
         client = gconf_client_get_default ();
 
         if (enabled) {
-                gconf_client_set_float (client, KEY_FONT_DPI, DPI_LARGER_FONT, NULL);
+                gdouble x_dpi;
+                gdouble u_dpi;
+
+                x_dpi = get_dpi_from_x_server ();
+                u_dpi = (double)DPI_FACTOR_LARGER * x_dpi;
+
+                g_debug ("GsdA11yPreferences: setting x-dpi=%f user-dpi=%f", x_dpi, u_dpi);
+
+                gconf_client_set_float (client, KEY_FONT_DPI, u_dpi, NULL);
         } else {
                 gconf_client_unset (client, KEY_FONT_DPI, NULL);
         }
@@ -697,14 +730,15 @@ key_changed_cb (GConfClient              *client,
 
 static void
 setup_dialog (GsdA11yPreferencesDialog *dialog,
-              GladeXML                 *xml)
+              GtkBuilder               *builder)
 {
         GtkWidget   *widget;
         gboolean     enabled;
         gboolean     is_writable;
         GConfClient *client;
 
-        widget = glade_xml_get_widget (xml, "sticky_keys_checkbutton");
+        widget = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                     "sticky_keys_checkbutton"));
         dialog->priv->sticky_keys_checkbutton = widget;
         g_signal_connect (widget,
                           "toggled",
@@ -716,7 +750,8 @@ setup_dialog (GsdA11yPreferencesDialog *dialog,
                 gtk_widget_set_sensitive (widget, FALSE);
         }
 
-        widget = glade_xml_get_widget (xml, "bounce_keys_checkbutton");
+        widget = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                     "bounce_keys_checkbutton"));
         dialog->priv->bounce_keys_checkbutton = widget;
         g_signal_connect (widget,
                           "toggled",
@@ -728,7 +763,8 @@ setup_dialog (GsdA11yPreferencesDialog *dialog,
                 gtk_widget_set_sensitive (widget, FALSE);
         }
 
-        widget = glade_xml_get_widget (xml, "slow_keys_checkbutton");
+        widget = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                     "slow_keys_checkbutton"));
         dialog->priv->slow_keys_checkbutton = widget;
         g_signal_connect (widget,
                           "toggled",
@@ -740,7 +776,8 @@ setup_dialog (GsdA11yPreferencesDialog *dialog,
                 gtk_widget_set_sensitive (widget, FALSE);
         }
 
-        widget = glade_xml_get_widget (xml, "high_contrast_checkbutton");
+        widget = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                     "high_contrast_checkbutton"));
         dialog->priv->high_contrast_checkbutton = widget;
         g_signal_connect (widget,
                           "toggled",
@@ -752,7 +789,8 @@ setup_dialog (GsdA11yPreferencesDialog *dialog,
                 gtk_widget_set_sensitive (widget, FALSE);
         }
 
-        widget = glade_xml_get_widget (xml, "at_screen_keyboard_checkbutton");
+        widget = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                     "at_screen_keyboard_checkbutton"));
         dialog->priv->screen_keyboard_checkbutton = widget;
         g_signal_connect (widget,
                           "toggled",
@@ -770,7 +808,8 @@ setup_dialog (GsdA11yPreferencesDialog *dialog,
                 gtk_widget_hide (widget);
         }
 
-        widget = glade_xml_get_widget (xml, "at_screen_reader_checkbutton");
+        widget = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                     "at_screen_reader_checkbutton"));
         dialog->priv->screen_reader_checkbutton = widget;
         g_signal_connect (widget,
                           "toggled",
@@ -788,7 +827,8 @@ setup_dialog (GsdA11yPreferencesDialog *dialog,
                 gtk_widget_hide (widget);
         }
 
-        widget = glade_xml_get_widget (xml, "at_screen_magnifier_checkbutton");
+        widget = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                     "at_screen_magnifier_checkbutton"));
         dialog->priv->screen_magnifier_checkbutton = widget;
         g_signal_connect (widget,
                           "toggled",
@@ -806,7 +846,8 @@ setup_dialog (GsdA11yPreferencesDialog *dialog,
                 gtk_widget_hide (widget);
         }
 
-        widget = glade_xml_get_widget (xml, "large_print_checkbutton");
+        widget = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                     "large_print_checkbutton"));
         dialog->priv->large_print_checkbutton = widget;
         g_signal_connect (widget,
                           "toggled",
@@ -848,19 +889,33 @@ setup_dialog (GsdA11yPreferencesDialog *dialog,
 static void
 gsd_a11y_preferences_dialog_init (GsdA11yPreferencesDialog *dialog)
 {
-        GtkWidget *widget;
-        GladeXML  *xml;
+        static const gchar *ui_file_path = GTKBUILDERDIR "/" GTKBUILDER_UI_FILE;
+        gchar *objects[] = {"main_box", NULL};
+        GError *error = NULL;
+        GtkBuilder  *builder;
 
         dialog->priv = GSD_A11Y_PREFERENCES_DIALOG_GET_PRIVATE (dialog);
 
-        xml = glade_xml_new (GLADEDIR "/" GLADE_XML_FILE, "main_box", PACKAGE);
-        g_assert (xml != NULL);
+        builder = gtk_builder_new ();
+        gtk_builder_set_translation_domain (builder, PACKAGE);
+        if (gtk_builder_add_objects_from_file (builder, ui_file_path, objects,
+                                               &error) == 0) {
+                g_warning ("Could not load A11Y-UI: %s", error->message);
+                g_error_free (error);
+        } else {
+                GtkWidget *widget;
 
-        widget = glade_xml_get_widget (xml, "main_box");
-        gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), widget);
+                widget = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                             "main_box"));
+                gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox),
+                                   widget);
+                gtk_container_set_border_width (GTK_CONTAINER (widget), 12);
+                setup_dialog (dialog, builder);
+       }
+
+        g_object_unref (builder);
 
         gtk_container_set_border_width (GTK_CONTAINER (dialog), 12);
-        gtk_container_set_border_width (GTK_CONTAINER (widget), 12);
         gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
         gtk_window_set_title (GTK_WINDOW (dialog), _("Universal Access Preferences"));
         gtk_window_set_icon_name (GTK_WINDOW (dialog), "preferences-desktop-accessibility");
@@ -877,9 +932,6 @@ gsd_a11y_preferences_dialog_init (GsdA11yPreferencesDialog *dialog)
                           G_CALLBACK (on_response),
                           dialog);
 
-        setup_dialog (dialog, xml);
-
-        g_object_unref (xml);
 
         gtk_widget_show_all (GTK_WIDGET (dialog));
 }
