@@ -69,12 +69,6 @@
 # define EJECT_COMMAND "eject -T"
 #endif
 
-#if defined(__OpenBSD__) || defined(__FreeBSD__)
-# define SLEEP_COMMAND "zzz"
-#else
-# define SLEEP_COMMAND "apm"
-#endif
-
 #define GSD_MEDIA_KEYS_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_MEDIA_KEYS_MANAGER, GsdMediaKeysManagerPrivate))
 
 typedef struct {
@@ -267,7 +261,7 @@ static void
 dialog_init (GsdMediaKeysManager *manager)
 {
         if (manager->priv->dialog != NULL
-            && !gsd_media_keys_window_is_valid (GSD_MEDIA_KEYS_WINDOW (manager->priv->dialog))) {
+            && !gsd_osd_window_is_valid (GSD_OSD_WINDOW (manager->priv->dialog))) {
                 gtk_widget_destroy (manager->priv->dialog);
                 manager->priv->dialog = NULL;
         }
@@ -596,8 +590,9 @@ do_eject_action (GsdMediaKeysManager *manager)
         char *command;
 
         dialog_init (manager);
-        gsd_media_keys_window_set_action (GSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
-                                          GSD_MEDIA_KEYS_WINDOW_ACTION_EJECT);
+        gsd_media_keys_window_set_action_custom (GSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
+                                                 "media-eject",
+                                                 FALSE);
         dialog_show (manager);
 
         command = gconf_client_get_string (manager->priv->conf_client,
@@ -895,9 +890,6 @@ do_action (GsdMediaKeysManager *manager,
         case EMAIL_KEY:
                 do_mail_action (manager);
                 break;
-        case SLEEP_KEY:
-                do_sleep_action (SLEEP_COMMAND, "xset dpms force off");
-                break;
         case SCREENSAVER_KEY:
                 if ((cmd = g_find_program_in_path ("gnome-screensaver-command"))) {
                         execute (manager, "gnome-screensaver-command --lock", FALSE, FALSE);
